@@ -61,9 +61,10 @@ def feature(name):
 class Features(object):
     IF = feature("IF")
     ELIF = feature("ELIF")
-    STATIC_TYPES = feature("STATIC_TYPES")
     TYPE_ARRAY = feature("TYPE_ARRAY")
     RANDOM_VAR = feature("RANDOM_VAR")
+    FOR_LOOP = feature("FOR_LOOP")
+    FUNC = feature("FUNC")
 
-    ALL = IF | ELIF | TYPE_ARRAY | RANDOM_VAR
+    ALL = IF | ELIF | TYPE_ARRAY | RANDOM_VAR | FOR_LOOP | FUNC
     NONE = FeatureSet([])
